@@ -3,7 +3,7 @@
 """
 Object oriented lib to Open Office Presentations
 
-Copyright 2008 Matt Harrison
+Copyright 2008-2009 Matt Harrison
 Licensed under Apache License, Version 2.0 (current)
 """
 
@@ -15,8 +15,13 @@ import os
 import sys
 import tempfile
 
-import pygments
-from pygments import formatter, lexers
+try:
+    import pygments
+    from pygments import formatter, lexers
+    pygmentsAvail = True
+except:
+    print 'Could not import pygments code highlighting will not work'
+    pygmentsAvail = False
 import zipwrap
 import Image
 import imagescale
@@ -63,7 +68,7 @@ MONO_FONT = 'Courier New' # I like 'Envy Code R'
 NORMAL_FONT = 'Arial'
 
 SLIDE_WIDTH = 30 # cm
-SLIDE_HEIGHT = 21 
+SLIDE_HEIGHT = 21
 
 def cwd_decorator(func):
     """
@@ -81,7 +86,7 @@ def cwd_decorator(func):
             if directory:
                 os.chdir(directory)
         data = func(*args, **kw)
-        os.chdir(cur_dir)        
+        os.chdir(cur_dir)
         return data
     return wrapper
 
@@ -114,7 +119,9 @@ def sub_el(parent, tag, attrib=None):
 def to_xml(node):
     """ convert an etree node to xml """
     fout = sio.StringIO()
-    etree = PrefixedWriter(node)
+    #etree = PrefixedWriter(node)
+    etree = et.ElementTree(node)
+
     etree.write(fout)
     xml = fout.getvalue()
     return xml
@@ -142,7 +149,7 @@ class Preso(object):
         self.limit_pages = [] # can be list of page numbers (not indexes to export)
         self._pictures = [] # list of Picture instances
         self._footer_count = 0
-        # xml elements 
+        # xml elements
         self._root = None
         self._auto_styles = None
         self._presentation = None
@@ -150,14 +157,16 @@ class Preso(object):
         self._styles_added = {}
 
         self._init_xml()
-        
+        self.master_page_name_cover = None
+        self.master_page_name_normal = None
+
     def _init_xml(self):
         self._root = el('office:document-content', attrib=DOC_CONTENT_ATTRIB)
         o_scripts = sub_el(self._root, 'office:scripts')
         self._auto_styles = sub_el(self._root, 'office:automatic-styles')
         o_body = sub_el(self._root, 'office:body')
         self._presentation = sub_el(o_body, 'office:presentation')
-        
+
     def add_imported_auto_style(self, style_node):
         self._auto_styles.append(style_node)
         style_node.parent = self._auto_styles
@@ -176,15 +185,26 @@ class Preso(object):
             self.slides.append(XMLSlide(self, slide_xml, odp))
 
     def get_data(self, style_file=None):
+        if style_file and not os.path.exists(style_file):
+            print "template file %s doesn't exist" % style_file
+            assert False
         fd, filename = tempfile.mkstemp()
         zip_odp = self.to_file()
         if style_file:
             self.add_otp_style(zip_odp, style_file)
         zip_odp.zipit(filename)
         data = open(filename).read()
+        os.close(fd)
         os.remove(filename)
         return data
 
+    def set_template(self, style_file):
+        style = zipwrap.ZipWrap(style_file)
+        title_name, normal_name = list(self.get_master_page_names(style.cat('content.xml')))[:2]
+        self.master_page_name_cover = title_name
+        self.master_page_name_normal = normal_name
+
+
     def add_otp_style(self, zip_odp, style_file):
         """
         takes the slide content and merges in the style_file
@@ -192,11 +212,17 @@ class Preso(object):
         style = zipwrap.ZipWrap(style_file)
         if 'Pictures' in style.cat(''):
             for p in style.cat('Pictures'):
-                picture_file = 'Pictures/'+p 
+                picture_file = 'Pictures/'+p
                 zip_odp.touch(picture_file, style.cat(picture_file))
-        zip_odp.touch('styles.xml', style.cat('styles.xml'))
+        xml_data = style.cat('styles.xml')
+        zip_odp.touch('styles.xml', xml_data)
         return zip_odp
-                   
+
+    def get_master_page_names(self, xml_data):
+        root = et.fromstring(xml_data)
+        pages = root.findall('.//{urn:oasis:names:tc:opendocument:xmlns:drawing:1.0}page')
+        for page in pages:
+            yield page.get('{urn:oasis:names:tc:opendocument:xmlns:drawing:1.0}master-page-name')
 
     def to_file(self, filename=None):
         """
@@ -241,12 +267,12 @@ class Preso(object):
                 filetype = 'image/png'
             elif filename == 'Configurations2/':
                 filetype = 'application/vnd.sun.xml.ui.configuration'
-            
+
             content += """ <manifest:file-entry manifest:media-type="%s" manifest:full-path="%s"/> """ % (filetype, filename)
 
         content += """</manifest:manifest>"""
         return content
-        
+
 
     def meta_xml(self):
         return """<?xml version="1.0" encoding="UTF-8"?>
@@ -273,9 +299,11 @@ class Preso(object):
     def styles_xml(self):
         filename = os.path.join(DATA_DIR, 'styles.xml')
         data = open(filename).read()
+        data = data.decode('utf-8')
         if NORMAL_FONT != 'Arial':
-            data = data.replace('fo:font-family="Arial"',
-                                'fo:font-family="%s"' %NORMAL_FONT)
+            data = data.replace(u'fo:font-family="Arial"',
+                                u'fo:font-family="%s"' %NORMAL_FONT)
+        #if "Arial" in data:
         return data
 
     def to_xml(self):
@@ -298,8 +326,8 @@ class Preso(object):
         if name not in self._styles_added:
             self._styles_added[name] = 1
             self._auto_styles.append(node)
-            
-        
+
+
     def add_slide(self):
         pnum = len(self.slides)+1
         s = Slide(self, page_number=pnum)
@@ -315,7 +343,7 @@ class Preso(object):
         f.name = 'ftr%d'%(self._footer_count)
         self._footer_count += 1
         self.slides[-1].footer = f
-    
+
 
 class Animation(object):
     ANIM_COUNT = 1
@@ -328,7 +356,7 @@ class Animation(object):
         return my_id
 
     def get_node(self):
-        """   
+        """
 	    <anim:par smil:begin="next">
 	      <anim:par smil:begin="0s">
 		<anim:par smil:begin="0s" smil:fill="hold" presentation:node-type="on-click" presentation:preset-class="entrance" presentation:preset-id="ooo-entrance-appear">
@@ -353,7 +381,7 @@ class Animation(object):
                                                     'smil:to':'visible'})
         return par
 
-                 
+
 class ImportedPicture(object):
     """
     Pictures used when importing slides
@@ -361,7 +389,7 @@ class ImportedPicture(object):
     def __init__(self, name, data):
         self.internal_name = name
         self.data = data
-        
+
     def get_data(self):
         return self.data
 
@@ -397,7 +425,7 @@ class Picture(object):
 
     def update_frame_attributes(self, attrib):
         """ For positioning update the frame """
-            
+
         if 'align' in self.user_defined:
             align = self.user_defined['align']
             if 'top' in align:
@@ -414,12 +442,12 @@ class Picture(object):
         name = str(Picture.COUNT) + ext
         Picture.COUNT += 1
         return name
-    
+
     def get_xywh(self, measurement=None):
         if measurement is None or measurement == 'cm':
             measurement = 'cm'
             scale = Picture.CM_SCALE
-        
+
         DPCM = 1 # dots per cm
         if 'crop' in self.user_defined.get('classes', []):
             x,y,w,h = imagescale.adjust_crop(SLIDE_WIDTH*DPCM, SLIDE_HEIGHT*DPCM,self.w, self.h)
@@ -469,7 +497,7 @@ class XMLSlide(object):
 
     def page_num(self):
         """ not an int, usually 'Slide 1' or 'page1' """
-        name = self.page_node.attrib.get('{urn:oasis:names:tc:opendocument:xmlns:drawing:1.0}name', None) 
+        name = self.page_node.attrib.get('{urn:oasis:names:tc:opendocument:xmlns:drawing:1.0}name', None)
         return name
 
     def _mangle_name(self):
@@ -515,19 +543,20 @@ class XMLSlide(object):
                     # mangle name
                     node.attrib[attr_name] = self.mangled + attr_value
                     self.preso.add_imported_auto_style(node)
-        
-        
-        
-        
+
+
+
+
 
     def get_node(self):
-        return self.page_node                               
+        return self.page_node
+
+
 
-        
-                          
 class Slide(object):
     def __init__(self, preso, page_number=None):
         self.title_frame = None
+        self.preso = preso
         self.text_frames = []
         self._cur_text_frame = None
         self.pic_frame = None
@@ -556,11 +585,12 @@ class Slide(object):
 
     def insert_line_breaks(self):
         """
-        If you want to write out existing line breaks, but don't have content to write 
+        If you want to write out existing line breaks, but don't have content to write
         call this
         """
         if self.cur_element:
             self.cur_element.line_break()
+            #self.cur_element.write('')
 
     def start_animation(self, anim):
         self.animations.append(anim)
@@ -587,7 +617,7 @@ class Slide(object):
         if self.cur_element is None:
             self.add_text_frame()
         self.pending_styles.append(style)
-    
+
     def pop_style(self):
         popped = self.pending_styles.pop()
 
@@ -613,7 +643,7 @@ class Slide(object):
         """
         # pictures should be added the the draw:frame element
         self.pic_frame = PictureFrame(self, p)
-        self.pic_frame.add_node('draw:image', attrib={'xlink:href':os.path.join('Pictures', p.internal_name),
+        self.pic_frame.add_node('draw:image', attrib={'xlink:href': 'Pictures/' + p.internal_name,
                                                       'xlink:type':'simple',
                                                       'xlink:show':'embed',
                                                       'xlink:actuate':'onLoad' })
@@ -629,7 +659,7 @@ class Slide(object):
 
     def pop_element(self):
         self.cur_element = self.element_stack.pop()
-        
+
     def to_xml(self):
         node = self.get_node()
         return to_xml(node)
@@ -646,12 +676,23 @@ class Slide(object):
         ins = copy.copy(self)
         ins._fire_page_number(self.page_number+1)
         return ins
-    
+
+    def _get_master_page_name(self):
+        if self.page_number == 1:
+            if self.preso.master_page_name_cover:
+                return self.preso.master_page_name_cover
+            return 'Default'
+        else:
+            if self.preso.master_page_name_normal:
+                return self.preso.master_page_name_normal
+            return 'Default'
+
     def _init_xml(self):
+        mpn = self._get_master_page_name()
         self._page = el('draw:page', attrib={
                 'draw:name':'page%d' % self.page_number,
                 'draw:style-name':'dp1',
-                'draw:master-page-name':'Default',
+                'draw:master-page-name':self._get_master_page_name(),
                 'presentation:presentation-page-layout-name':'AL1T0'
                 })
         office_forms = sub_el(self._page, 'office:forms',
@@ -680,9 +721,10 @@ class Slide(object):
         if self.footer:
             self._page.attrib['presentation:use-footer-name'] = self.footer.name
         return self._page
-        
-    def add_text_frame(self):
-        self.text_frames.append(TextFrame(self))
+
+    def add_text_frame(self, attrib=None):
+        # should adjust width, x based on if existing boxes
+        self.text_frames.append(TextFrame(self, attrib))
         node = self.text_frames[-1].get_node()
         self._page.append(node)
         node.parent = self._page
@@ -720,7 +762,15 @@ class Slide(object):
             self._preso._auto_styles.append(et.fromstring(bl.default_styles())[0])
         self.cur_element = bl
 
-            
+    def add_table(self, t):
+        """
+        remember to call pop_element after done with table
+        """
+        self.push_element()
+        self._page.append(t.node)
+        t.node.parent = self._page
+        self.cur_element = t
+
     def write(self, text, **kw):
         if self.cur_element is None:
             self.add_text_frame()
@@ -737,9 +787,9 @@ class Slide(object):
         self.cur_element.pop_node()
 
     def parent_of(self, name):
-        """ 
+        """
         like pop_node, but traverse up parents.  When you find a node
-        with name, set cur_node to that 
+        with name, set cur_node to that
         """
         if self.cur_element:
             self.cur_element.parent_of(name)
@@ -757,7 +807,7 @@ class MixedContent(object):
         self.dirty = False # keep track if we have been written to
 
     def parent_of(self, name):
-        """ 
+        """
         go to parent of node with name, and set as cur_node.  Useful
         for creating new paragraphs
        """
@@ -784,7 +834,7 @@ class MixedContent(object):
             result = self._is_node(tagname, attributes, node=children[-1])
             return result
         return False
-        
+
     def _is_node(self, tagname, attributes=None, node=None):
         if node is None:
             node = self.cur_node
@@ -818,7 +868,7 @@ class MixedContent(object):
     def append(self, node):
         self.cur_node.append(node)
         node.parent = self.cur_node
-    
+
     def _check_add_node(self, parent, name):
         ''' Returns False if bad to make name a child of parent '''
 
@@ -826,18 +876,19 @@ class MixedContent(object):
             if parent.tag == 'draw:text-box':
                 return False
         return True
-        
+
     def _add_node(self, parent, name, attrib):
         if not self._check_add_node(parent, name):
             raise Exception, 'Bad child (%s) for %s)' %(name, parent.tag)
         new_node = sub_el(parent, name, attrib)
         return new_node
-        
+
     def add_node(self, node_name, attrib=None):
         if attrib is None:
             attrib = {}
         new_node = self._add_node(self.cur_node, node_name, attrib)
         self.cur_node = new_node
+        return self.cur_node
 
     def pop_node(self):
         if self.cur_node.parent == self.node:
@@ -848,6 +899,9 @@ class MixedContent(object):
         self.cur_node = self.cur_node.parent
 
     def _add_styles(self, add_paragraph=True, add_text=True):
+        """
+        Adds paragraph and span wrappers if necessary based on style
+        """
         p_styles = {'fo:text-align':self._default_align}
         t_styles = {}
         for s in self.slide.pending_styles:
@@ -855,16 +909,17 @@ class MixedContent(object):
                 p_styles.update(s.styles)
             elif isinstance(s, TextStyle):
                 t_styles.update(s.styles)
-                
-        para = ParagraphStyle(**p_styles)     
+
+        para = ParagraphStyle(**p_styles)
 
         if add_paragraph or self.slide.paragraph_attribs:
             p_attrib = {'text:style-name':para.name}
             p_attrib.update(self.slide.paragraph_attribs)
-            if self._is_last_child('text:p', p_attrib):
-                children = self.cur_node.getchildren()
-                self.cur_node = children[-1] 
-            elif not self._in_tag('text:p', p_attrib):
+            # if self._is_last_child('text:p', p_attrib):
+            #     children = self.cur_node.getchildren()
+            #     self.cur_node = children[-1]
+            #elif not self._in_tag('text:p', p_attrib):
+            if not self._in_tag('text:p', p_attrib):
                 self.parent_of('text:p')
                 # Create paragraph style first
                 self.slide._preso.add_style(para)
@@ -881,7 +936,7 @@ class MixedContent(object):
                   last.attrib['text:style-name'] == text.name and \
                   last.tail is None: # if we have a tail, we can't reuse
                     self.cur_node = children[-1]
-                    return 
+                    return
             if not self._is_node('text:span', {'text:style-name':text.name}):
                 # Create text style
                 self.slide._preso.add_style(text)
@@ -891,7 +946,6 @@ class MixedContent(object):
     def _add_pending_nodes(self):
         for node, attr in self.pending_nodes:
             self.add_node(node, attr)
-            
 
     def line_break(self):
         """insert as many line breaks as the insert_line_break variable says
@@ -902,14 +956,18 @@ class MixedContent(object):
                 # we can just add a text:p and no line-break
                 # Create paragraph style first
                 self.add_node('text:p')
-            else:
-                self.add_node('text:line-break')
+            #else:
+            self.add_node('text:line-break')
+            self.pop_node()
+            if self.cur_node.tag == 'text:p':
+                return
             if self.cur_node.parent.tag != 'text:p':
                 self.pop_node()
 
         self.slide.insert_line_break = 0
 
 
+
     def write(self, text, add_p_style=True, add_t_style=True):
         """
         see mixed content
@@ -919,12 +977,12 @@ class MixedContent(object):
         white spaces then dealing with the '' (empty strings) which
         would be the extra spaces
         """
-        self.line_break()
         self._add_styles(add_p_style, add_t_style)
+        #self.line_break()
         self._add_pending_nodes()
 
         spaces = []
-        for i, letter in enumerate(text): 
+        for i, letter in enumerate(text):
             if letter == ' ':
                 spaces.append(letter)
                 continue
@@ -962,7 +1020,7 @@ class MixedContent(object):
             else:
                 self.add_node('text:s')
             self.pop_node()
-            
+
 
     def _write(self, letter):
         children = self.cur_node.getchildren()
@@ -970,7 +1028,7 @@ class MixedContent(object):
             child = children[-1]
             cur_text = child.tail or ''
             child.tail = cur_text + letter
-        else:    
+        else:
             cur_text = self.cur_node.text or ''
             self.cur_node.text = cur_text + letter
         self.dirty = True
@@ -986,8 +1044,8 @@ class Footer(MixedContent):
             raise Exception("set footer name")
         self.node.attrib['presentation:name'] = self.name
         return self.node
-                                                      
-        
+
+
 
 class PictureFrame(MixedContent):
     def __init__(self, slide, picture, attrib=None):
@@ -1003,17 +1061,18 @@ class PictureFrame(MixedContent):
             }
         attrib = picture.update_frame_attributes(attrib)
         MixedContent.__init__(self, slide, 'draw:frame', attrib=attrib)
-        
+
 
 class TextFrame(MixedContent):
     def __init__(self, slide, attrib=None):
         attrib = attrib or {
             'presentation:style-name':'pr2',
             'draw:layer':'layout',
-            'svg:width':'25.199cm',
-            'svg:height':'13.86cm',
-            'svg:x':'1.4cm',
-            'svg:y':'4.577cm',
+            'svg:width':'%.2fcm' % (SLIDE_WIDTH*.84), #'25.199cm',
+            'svg:height':'%.2fcm' % (SLIDE_HEIGHT*.66), #'13.86cm',
+            'svg:x':'%.2fcm' % ((SLIDE_WIDTH - (SLIDE_WIDTH*.84))/2),#'1.4cm',
+            #'svg:y':'%.2fcm' % ((SLIDE_HEIGHT - (SLIDE_HEIGHT*.66))/2),#'4.577cm',
+            'svg:y':'%.2fcm' % (SLIDE_HEIGHT*.25),
             'presentation:class':'subtitle'
             }
 
@@ -1027,26 +1086,26 @@ class TextFrame(MixedContent):
 
     def to_xml(self):
         return to_xml(self.get_node())
-    
+
     def _in_bullet(self):
         return self._in_tag('text:list')
 
-        
+
 class TitleFrame(TextFrame):
     def __init__(self, slide, attrib=None):
         attrib = attrib or {
             'presentation:style-name':'Default-title',
             'draw:layer':'layout',
-            'svg:width':'25.199cm',
-            'svg:height':'1.737cm',
-            'svg:x':'1.4cm',
-            'svg:y':'1.721cm',
+            'svg:width':'%.2fcm' % (SLIDE_WIDTH*.84), #'25.199cm',
+            'svg:height':'%.2fcm' % (SLIDE_HEIGHT*.087), #'svg:height':'1.737cm',
+            'svg:x':'%.2fcm' % ((SLIDE_WIDTH - (SLIDE_WIDTH*.84))/2),#'1.4cm'
+            'svg:y':'%.2fcm' % (SLIDE_HEIGHT*.087), #'1.721cm',
             'presentation:class':'title'
             }
 
         TextFrame.__init__(self, slide, attrib)
-        self._default_align = 'center'        
-        
+        self._default_align = 'center'
+
 class NotesFrame(TextFrame):
     def __init__(self, slide, attrib=None):
         attrib = attrib or {
@@ -1061,8 +1120,8 @@ class NotesFrame(TextFrame):
             }
         TextFrame.__init__(self, slide, attrib)
         self._preso_notes = el('presentation:notes', attrib={'draw:style-name':'dp2'})
-        self._page_thumbnail = sub_el(self._preso_notes, 
-                                      'draw:page-thumbnail', 
+        self._page_thumbnail = sub_el(self._preso_notes,
+                                      'draw:page-thumbnail',
                                       attrib={
                 'presentation:style-name':'gr1',
                 'draw:layer':'layout',
@@ -1074,17 +1133,17 @@ class NotesFrame(TextFrame):
                 'presentation:class':'page'})
         self._preso_notes.append(self.node)
         self.node.parent = self._preso_notes
-        
+
     def new_page_num(self, new_num):
         self._page_thumbnail.attrib['draw:page-number']='%d'%new_num
 
     def get_node(self):
         return self._preso_notes
-        
+
 
 class TextStyle(object):
     """
-    based on 
+    based on
     http://books.evc-cit.info/odbook/ch03.html#char-para-styling-section
     """
     font_weight = dict(
@@ -1145,7 +1204,7 @@ class TextStyle(object):
         LINE_HEIGHT = 'line-height',
         FIXED = 'fixed'
         )
-    
+
     FAMILY = 'text'
     STYLE_PROP = 'style:text-properties'
     PREFIX = 'T%d'
@@ -1173,10 +1232,10 @@ class TextStyle(object):
     def style_node(self, additional_style_attrib=None):
         """
         generate a style node (for automatic-styles)
-        
+
         could specify additional attributes such as
         'style:parent-style-name' or 'style:list-style-name'
-       
+
         """
         style_attrib = {'style:name':self.name,
                         'style:family':self.FAMILY}
@@ -1193,61 +1252,73 @@ class ParagraphStyle(TextStyle):
     text_align = dict(
         START = 'start',
         END = 'end',
-        CENTER = 'center', 
+        CENTER = 'center',
         JUSTIFY = 'justify'
         )
-        
+
     FAMILY = 'paragraph'
     STYLE_PROP = 'style:paragraph-properties'
     PREFIX = 'P%d'
 
 
-class OdtCodeFormatter(formatter.Formatter):  
-    def __init__(self, writable, preso):
-        formatter.Formatter.__init__(self)
-        self.writable = writable
-        self.preso = preso
+if pygmentsAvail:
 
-    def format(self, source, outfile):
-        tclass = pygments.token.Token
-        for ttype, value in source:
-            # getting ttype, values like (Token.Keyword.Namespace, u'')
-            if value == '':
-                continue
-            style_attrib = self.get_style(ttype)
-            tstyle = TextStyle(**style_attrib)
-            self.writable.slide.push_style(tstyle)
-            if value == '\n':
-                self.writable.slide.insert_line_break = 1
-                self.writable.line_break()
-            else:
-                parts = value.split('\n')
-                for part in parts[:-1]:
-                    self.writable.write(part)
+    class OdtCodeFormatter(formatter.Formatter):
+        def __init__(self, writable, preso):
+            formatter.Formatter.__init__(self)
+            self.writable = writable
+            self.preso = preso
+
+        def format(self, source, outfile):
+            tclass = pygments.token.Token
+            # push default style
+            default_style_attrib = self.get_style(tclass.Text)
+            self.writable.slide.push_style(TextStyle(**default_style_attrib))
+            for ttype, value in source:
+                pop = True
+                # getting ttype, values like (Token.Keyword.Namespace, u'')
+                if value == '':
+                    continue
+                style_attrib = self.get_style(ttype)
+                tstyle = TextStyle(**style_attrib)
+                self.writable.slide.push_style(tstyle)
+                if value == '\n':
                     self.writable.slide.insert_line_break = 1
-                    self.writable.line_break()
-                self.writable.write(parts[-1])
+                    self.writable.write('') # will insert break/formatting
+                    self.writable.slide.insert_line_breaks()
+                    # when we only put a line break, we don't want to pop cause we never put in a span to write to
+                    pop = False
+                else:
+                    parts = value.split('\n')
+                    for part in parts[:-1]:
+                        self.writable.write(part)
+                        self.writable.slide.insert_line_break = 1
+                        self.writable.slide.insert_line_breaks()
+                        #self.writable.write('') #insert break
+                    self.writable.write(parts[-1])
+
+                self.writable.slide.pop_style()
+                if pop:
+                    self.writable.pop_node()
             self.writable.slide.pop_style()
-            self.writable.pop_node()
+
+        def get_style(self, tokentype):
+            while not self.style.styles_token(tokentype):
+                tokentype = tokentype.parent
+            value = self.style.style_for_token(tokentype)
+            # default to monospace
+            results = {
+                'fo:font-family':MONO_FONT,
+                'style:font-family-generic':"swiss",
+                'style:font-pitch':"fixed"}
+            if value['color']:
+                results['fo:color'] = '#' + value['color']
+            if value['bold']:
+                results['fo:font-weight'] = 'bold'
+            if value['italic']:
+                results['fo:font-weight'] = 'italic'
+            return results
 
-            
-    def get_style(self, tokentype):
-        while not self.style.styles_token(tokentype):
-            tokentype = tokentype.parent
-        value = self.style.style_for_token(tokentype)
-        # default to monospace
-        results = {
-            'fo:font-family':MONO_FONT,
-            'style:font-family-generic':"swiss",
-            'style:font-pitch':"fixed"}
-        if value['color']:
-            results['fo:color'] = '#' + value['color']
-        if value['bold']:
-            results['fo:font-weight'] = 'bold'
-        if value['italic']:
-            results['fo:font-weight'] = 'italic'
-        return results
-          
 
 class OutlineList(MixedContent):
     """
@@ -1272,7 +1343,7 @@ class OutlineList(MixedContent):
 http://books.evc-cit.info/odbook/ch03.html#bulleted-numbered-lists-section
 
     Bonafide OOo output looks like this:
-      
+
  <draw:text-box>
 	    <text:list text:style-name="L2">
 	      <text:list-item>
@@ -1301,9 +1372,7 @@ http://books.evc-cit.info/odbook/ch03.html#bulleted-numbered-lists-section
         self._default_align = 'start'
         self.attrib = attrib or {'text:style-name':'L2'}
         MixedContent.__init__(self, slide, 'text:list', attrib=self.attrib)
-        # if slide already has text insert line break
-        self.line_break()
-
+        self.slide.insert_line_break = 0
         self.parents = [self.node]
         self.level = 0
         self.style_file = 'auto_list.xml'
@@ -1339,10 +1408,108 @@ class NumberList(OutlineList):
         self.style_file = 'number_list.xml'
         self.style_name = 'number-list'
 
+
+class TableFrame(MixedContent):
+    """
+    Tables look like this:
+    <draw:frame draw:style-name="standard" draw:layer="layout"
+                  svg:width="14.098cm" svg:height="1.943cm"
+                  svg:x="7.01cm" svg:y="10.44cm">
+        <table:table table:template-name="default"
+                     table:use-first-row-styles="true"
+                     table:use-banding-rows-styles="true">
+          <table:table-column table:style-name="co1"/>
+          <table:table-column table:style-name="co1"/>
+          <table:table-column table:style-name="co1"/>
+          <table:table-column table:style-name="co1"/>
+          <table:table-column table:style-name="co2"/>
+          <table:table-row table:style-name="ro1"
+                           table:default-cell-style-name="ce1">
+            <table:table-cell>
+              <text:p>Header
+              </text:p>
+            </table:table-cell>
+            <table:table-cell>
+              <text:p>2
+              </text:p>
+            </table:table-cell>
+            <table:table-cell>
+              <text:p>3
+              </text:p>
+            </table:table-cell>
+            <table:table-cell>
+              <text:p>4
+              </text:p>
+            </table:table-cell>
+            <table:table-cell>
+              <text:p>5
+              </text:p>
+            </table:table-cell>
+          </table:table-row>
+          <table:table-row table:style-name="ro1"
+                           table:default-cell-style-name="ce1">
+            <table:table-cell>
+              <text:p>row1
+              </text:p>
+            </table:table-cell>
+            <table:table-cell>
+              <text:p>2
+              </text:p>
+            </table:table-cell>
+            <table:table-cell>
+              <text:p>3
+              </text:p>
+            </table:table-cell>
+            <table:table-cell>
+              <text:p>4
+              </text:p>
+            </table:table-cell>
+            <table:table-cell>
+              <text:p>5
+              </text:p>
+            </table:table-cell>
+          </table:table-row>
+        </table:table>
+        <draw:image xlink:href="Pictures/TablePreview1.svm"
+        xlink:type="simple" xlink:show="embed"
+                    xlink:actuate="onLoad"/>
+      </draw:frame>
+    """
+    def __init__(self, slide, frame_attrib=None, table_attrib=None):
+        self.frame_attrib = frame_attrib or {'draw:style-name':'standard',
+                                             'draw:layer':'layout',
+                                             'svg:width':'%.2fcm' % (SLIDE_WIDTH*.84), #'25.199cm',
+                                             #'svg:height':'13.86cm', #'''1.943cm',
+                                             'svg:x':'%.2fcm' % ((SLIDE_WIDTH - (SLIDE_WIDTH*.84))/2),#'1.4cm',
+                                             'svg:y':'147pt'#'24.577m'
+                                             }
+        MixedContent.__init__(self, slide, 'draw:frame', attrib=self.frame_attrib)
+
+        self.attrib = table_attrib or {'table:template-name':'default',
+                                 'table:use-first-row-styles':'true',
+                                 'table:use-banding-rows-styles':'true'}
+        self.table = self.add_node('table:table', attrib=self.attrib)
+        self.row = None
+
+    def add_row(self, attrib=None):
+        # rows always go on the table:table elem
+        self.cur_node = self.table
+        attrib = attrib or {'table:style-name':'ro1',
+                            'table:default-cell-style-name':'ce1'}
+        self.add_node( 'table:table-row', attrib)
+
+    def add_cell(self, attrib=None):
+        self.slide.insert_line_break = 0
+        if self._in_tag('table:table-cell'):
+            self.parent_of('table:table-cell')
+        elif not self._in_tag('table:table-row'):
+            self.add_row()
+        self.add_node('table:table-cell', attrib)
+
 def _test():
     import doctest
     doctest.testmod()
 
 if __name__ == '__main__':
     _test()
-        
+
